@@ -3,9 +3,18 @@ package com.akdeniz.googleplaycrawler;
 import java.io.IOException;
 import java.io.InputStream;
 import java.math.BigInteger;
+import java.security.Key;
+import java.security.KeyFactory;
+import java.security.MessageDigest;
+import java.security.PublicKey;
+import java.security.spec.RSAPublicKeySpec;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
+import java.util.PropertyResourceBundle;
+import java.util.ResourceBundle;
+
+import javax.crypto.Cipher;
 
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpResponse;
@@ -39,6 +48,7 @@ import com.akdeniz.googleplaycrawler.GooglePlay.ReviewResponse;
 import com.akdeniz.googleplaycrawler.GooglePlay.SearchResponse;
 import com.akdeniz.googleplaycrawler.GooglePlay.UploadDeviceConfigRequest;
 import com.akdeniz.googleplaycrawler.GooglePlay.UploadDeviceConfigResponse;
+import com.akdeniz.googleplaycrawler.misc.Base64;
 
 /**
  * This class provides
@@ -102,6 +112,7 @@ public class GooglePlayAPI {
 	private HttpClient client;
 	private String securityToken;
 	private String localization;
+	private String useragent;
 
 	/**
 	 * Default constructor. ANDROID ID and Authentication token must be supplied
@@ -127,6 +138,8 @@ public class GooglePlayAPI {
 		this.setEmail(email);
 		this.password = password;
 		setClient(new DefaultHttpClient(getConnectionManager()));
+		// setUseragent("Android-Finsky/3.10.14 (api=3,versionCode=8016014,sdk=15,device=GT-I9300,hardware=aries,product=GT-I9300)");
+		setUseragent("Android-Finsky/6.5.08.D-all (versionCode=80650800,sdk=23,device=noblelte,hardware=noblelte,product=noblelte,build=MMB29K:user)");
 	}
 
 	/**
@@ -160,6 +173,8 @@ public class GooglePlayAPI {
 		setSecurityToken((BigInteger.valueOf(checkinResponse.getSecurityToken()).toString(16)));
 
 		String c2dmAuth = loginAC2DM();
+		//login();
+		//String c2dmAuth= getToken();
 
 		AndroidCheckinRequest.Builder checkInbuilder = AndroidCheckinRequest.newBuilder(Utils
 				.generateAndroidCheckinRequest());
@@ -171,6 +186,61 @@ public class GooglePlayAPI {
 		// this is the second checkin to match credentials with android-id
 		return postCheckin(build.toByteArray());
 	}
+	
+	private static int readInt(byte[] bArr, int i) {
+		return (((((bArr[i] & 255) << 24) | 0) | ((bArr[i + 1] & 255) << 16)) | ((bArr[i + 2] & 255) << 8))
+				| (bArr[i + 3] & 255);
+	}
+  public static PublicKey createKeyFromString(String str, byte[] bArr) {
+    try {
+        byte[] decode = Base64.decode(str, 0);
+        int readInt = readInt(decode, 0);
+        byte[] obj = new byte[readInt];
+        System.arraycopy(decode, 4, obj, 0, readInt);
+        BigInteger bigInteger = new BigInteger(1, obj);
+        int readInt2 = readInt(decode, readInt + 4);
+        byte[] obj2 = new byte[readInt2];
+        System.arraycopy(decode, readInt + 8, obj2, 0, readInt2);
+        BigInteger bigInteger2 = new BigInteger(1, obj2);
+        decode = MessageDigest.getInstance("SHA-1").digest(decode);
+        bArr[0] = (byte) 0;
+        System.arraycopy(decode, 0, bArr, 1, 4);
+        return KeyFactory.getInstance("RSA").generatePublic(new RSAPublicKeySpec(bigInteger, bigInteger2));
+    } catch (Throwable e) {
+        throw new RuntimeException(e);
+    }
+}
+
+private static String encryptString(String str) {
+int i = 0;
+ResourceBundle bundle = PropertyResourceBundle.getBundle("com.akdeniz.googleplaycrawler.crypt");
+String string=bundle.getString("key");
+
+byte[] obj = new byte[5];
+Key createKeyFromString = createKeyFromString(string, obj);
+if (createKeyFromString == null) {
+	return null;
+}
+try {
+	Cipher instance = Cipher
+			.getInstance("RSA/ECB/OAEPWITHSHA1ANDMGF1PADDING");
+	byte[] bytes = str.getBytes("UTF-8");
+	int length = ((bytes.length - 1) / 86) + 1;
+	byte[] obj2 = new byte[(length * 133)];
+	while (i < length) {
+		instance.init(1, createKeyFromString);
+		byte[] doFinal = instance.doFinal(bytes, i * 86, i == length
+				+ -1 ? bytes.length - (i * 86) : 86);
+		System.arraycopy(obj, 0, obj2, i * 133, obj.length);
+		System.arraycopy(doFinal, 0, obj2, (i * 133) + obj.length,
+				doFinal.length);
+		i++;
+	}
+	return Base64.encodeToString(obj2, 10);
+} catch (Throwable e) {
+	throw new RuntimeException(e);
+}
+}
 
 	/**
 	 * Logins AC2DM server and returns authentication string.
@@ -179,7 +249,8 @@ public class GooglePlayAPI {
 		HttpEntity c2dmResponseEntity = executePost(URL_LOGIN,
 				new String[][] {
 						{ "Email", this.getEmail() },
-						{ "Passwd", this.password },
+						{ "EncryptedPasswd", encryptString(this.getEmail()+"\u0000"+this.password) },
+						{ "add_account", "1"},
 						{ "service", "ac2dm" },
 						{ "accountType", ACCOUNT_TYPE_HOSTED_OR_GOOGLE },
 						{ "has_permission", "1" },
@@ -188,7 +259,7 @@ public class GooglePlayAPI {
 						{ "device_country", "us" },
 						{ "device_country", "us" },
 						{ "lang", "en" },
-						{ "sdk_version", "21" }, }, null);
+						{ "sdk_version", "17" }, }, null);
 
 		Map<String, String> c2dmAuth = Utils.parseResponse(new String(Utils.readAll(c2dmResponseEntity
 				.getContent())));
@@ -225,8 +296,9 @@ public class GooglePlayAPI {
 
 		HttpEntity responseEntity = executePost(URL_LOGIN, new String[][] {
 				{ "Email", this.getEmail() },
-				{ "Passwd", this.password },
+				{ "EncryptedPasswd", encryptString(this.getEmail()+"\u0000"+this.password) },
 				{ "service", "androidmarket" },
+				{ "add_account", "1"},
 				{ "accountType", ACCOUNT_TYPE_HOSTED_OR_GOOGLE },
 				{ "has_permission", "1" },
 				{ "source", "android" },
@@ -234,7 +306,7 @@ public class GooglePlayAPI {
 				{ "app", "com.android.vending" },
 				{ "device_country", "en" },
 				{ "lang", "en" },
-				{ "sdk_version", "21" }, }, null);
+				{ "sdk_version", "17" }, }, null);
 
 		Map<String, String> response = Utils.parseResponse(new String(Utils.readAll(responseEntity
 				.getContent())));
@@ -598,9 +670,7 @@ public class GooglePlayAPI {
 						"nocache:billing.use_charging_poller,market_emails,buyer_currency,prod_baseline,checkin.set_asset_paid_app_field,shekel_test,content_ratings,buyer_currency_in_app,nocache:encrypted_apk,recent_changes" },
 				{ "X-DFE-Device-Id", this.getAndroidID() },
 				{ "X-DFE-Client-Id", "am-android-google" },
-				{
-						"User-Agent",
-						"Android-Finsky/3.10.14 (api=3,versionCode=8016014,sdk=15,device=GT-I9300,hardware=aries,product=GT-I9300)" },
+				{ "User-Agent", getUseragent() },
 				{ "X-DFE-SmallestScreenWidthDp", "320" },
 				{ "X-DFE-Filter-Level", "3" },
 				{ "Host", "android.clients.google.com" },
@@ -678,4 +748,19 @@ public class GooglePlayAPI {
 		this.localization = localization;
 	}
 
+	/**
+	 * @return the useragent
+	 */
+	public String getUseragent() {
+		return useragent;
+	}
+
+	/**
+	 * @param useragent
+	 *          the useragent to set
+	 */
+	public void setUseragent(String useragent) {
+		this.useragent = useragent;
+	}
+
 }
